@@ -4,6 +4,7 @@ Created on 11 jan. 2013
 @author: sander
 '''
 from abc import abstractmethod, ABCMeta
+from pylisp.utils.represent import represent
 
 
 class ProtocolElement(object):
@@ -16,11 +17,7 @@ class ProtocolElement(object):
         '''
 
     def __repr__(self):
-        # This works as long as we accept all properties as paramters in the
-        # constructor
-        params = ['%s=%r' % (k, v) for k, v in self.__dict__.iteritems()]
-        return '%s(%s)' % (self.__class__.__name__,
-                           ', '.join(params))
+        return represent(self.__class__.__name__, self.__dict__)
 
     def __str__(self):
         return str(self.to_bytes())
@@ -49,8 +46,6 @@ class ProtocolElement(object):
 
 
 class Protocol(ProtocolElement):
-    __metaclass__ = ABCMeta
-
     header_type = None
 
     @abstractmethod
@@ -61,3 +56,17 @@ class Protocol(ProtocolElement):
         super(Protocol, self).__init__()
         self.next_header = next_header
         self.payload = payload
+
+    @abstractmethod
+    def sanitize(self):
+        '''
+        Check and optionally fix properties
+        '''
+        # Let the parent do its stuff
+        super(Protocol, self).sanitize()
+
+        # Check if the next header is of the right type, and fix this header
+        # if we know better (i.e. the payload is a ProtocolElement so we know
+        # the header_type)
+        if isinstance(self.payload, Protocol):
+            self.next_header = self.payload.header_type
