@@ -3,10 +3,12 @@ Created on 6 jan. 2013
 
 @author: sander
 '''
-from IPy import IP
 from bitstring import ConstBitStream, BitArray, Bits
-from pylisp.utils.afi import read_afi_address_from_bitstream, \
-    get_bitstream_for_afi_address
+from ipaddress import IPv4Address, IPv6Address, IPv4Network, IPv6Network
+from pylisp.application.lispd.utils.prefix import determine_instance_id_and_afi
+from pylisp.utils.afi import read_afi_address_from_bitstream, get_bitstream_for_afi_address
+from pylisp.utils.lcaf.base import LCAFAddress
+from pylisp.utils.represent import represent
 import numbers
 
 
@@ -16,7 +18,7 @@ __all__ = ['LocatorRecord']
 class LocatorRecord(object):
     def __init__(self, priority=255, weight=0, m_priority=255, m_weight=0,
                  local=False, probed_locator=False, reachable=False,
-                 locator=None):
+                 address=None):
         '''
         Constructor
         '''
@@ -28,14 +30,19 @@ class LocatorRecord(object):
         self.local = local
         self.probed_locator = probed_locator
         self.reachable = reachable
-        self.locator = locator
+        self.address = address
+
+        # Store space for reserved bits
+        self._reserved1 = BitArray(13)
+
+    @staticmethod
+    def sort_key(locator):
+        # Provide a key that can be used for sorting
+        dummy, dummy, address = determine_instance_id_and_afi(locator.address)
+        return int(address)
 
     def __repr__(self):
-        # This works as long as we accept all properties as paramters in the
-        # constructor
-        params = ['%s=%r' % (k, v) for k, v in self.__dict__.iteritems()]
-        return '%s(%s)' % (self.__class__.__name__,
-                           ', '.join(params))
+        return represent(self.__class__.__name__, self.__dict__)
 
     def sanitize(self):
         '''
@@ -125,23 +132,32 @@ class LocatorRecord(object):
         # source RLOC MUST NOT be a multicast address.  The destination RLOC
         # SHOULD be a multicast address if it is being mapped from a
         # multicast destination EID.
-        #
-        # NOTE: where does source/destination RLOC usage come from in this
-        #       context?
-        # NOTE: how does it even make sense to use multicast addresses as
-        #       RLOCs?
-        if not isinstance(self.locator, IP) \
-        or self.locator.version() not in (4, 6):
-            raise ValueError('Locator must be IPv4 or IPv6 address')
 
-        if self.locator == IP('255.255.255.255'):
-            raise ValueError('Locator must not be broadcast address')
+        if isinstance(self.address, (IPv4Address, IPv6Address)):
+            addresses = [self.address]
+        elif isinstance(self.address, LCAFAddress):
+            addresses = self.address.get_addresses()
+        else:
+            raise ValueError('Locator must be an (LCAF) IPv4 or IPv6 address')
 
-        if self.locator in IP('224.0.0.0/24') \
-        or self.locator in IP('ff02::/16') \
-        or self.locator in IP('ff12::/16'):
-            raise ValueError('Locator must not be link-local multicast ' +
-                             'address')
+        for address in addresses:
+            if isinstance(self.address, IPv4Address):
+                if address == IPv4Address(u'255.255.255.255'):
+                    raise ValueError('Locator must not be the broadcast '
+                                     'address')
+
+                if address in IPv4Network(u'224.0.0.0/24'):
+                    raise ValueError('Locator must not be a link-local '
+                                     'multicast address')
+
+            elif isinstance(self.address, IPv6Address):
+                if address in IPv6Network(u'ff02::/16') \
+                or address in IPv6Network(u'ff12::/16'):
+                    raise ValueError('Locator must not be a link-local '
+                                     'multicast address')
+
+            else:
+                raise ValueError('Locator must be an IPv4 or IPv6 address')
 
     @classmethod
     def from_bytes(cls, bitstream):
@@ -161,8 +177,8 @@ class LocatorRecord(object):
         (record.priority, record.weight, record.m_priority,
          record.m_weight) = bitstream.readlist('4*uint:8')
 
-        # Skip over unused flags
-        bitstream.read(13)
+        # Read over unused flags
+        record._reserved1 = bitstream.read(13)
 
         # Read the flags
         (record.local,
@@ -170,7 +186,7 @@ class LocatorRecord(object):
          record.reachable) = bitstream.readlist('3*bool')
 
         # Read the locator
-        record.locator = read_afi_address_from_bitstream(bitstream)
+        record.address = read_afi_address_from_bitstream(bitstream)
 
         # Verify that the properties make sense
         record.sanitize()
@@ -198,7 +214,7 @@ class LocatorRecord(object):
                                             self.m_weight))
 
         # Add padding
-        bitstream += BitArray(13)
+        bitstream += self._reserved1
 
         # Add the flags
         bitstream += BitArray('bool=%d, bool=%d, bool=%d'
@@ -207,6 +223,6 @@ class LocatorRecord(object):
                                  self.reachable))
 
         # Add the locator
-        bitstream += get_bitstream_for_afi_address(self.locator)
+        bitstream += get_bitstream_for_afi_address(self.address)
 
         return bitstream
