@@ -3,11 +3,12 @@ Created on 6 jan. 2013
 
 @author: sander
 '''
-from IPy import IP
 from bitstring import ConstBitStream, BitArray, Bits
+from ipaddress import IPv4Network, IPv6Network
 from pylisp.packet.lisp.control import LocatorRecord
 from pylisp.utils.afi import read_afi_address_from_bitstream, \
     get_bitstream_for_afi_address
+from pylisp.utils.represent import represent
 import numbers
 
 
@@ -49,12 +50,11 @@ class MapReplyRecord(object):
         self.eid_prefix = eid_prefix
         self.locator_records = locator_records or []
 
+        # Store space for reserved bits
+        self._reserved1 = BitArray(12 + 4)
+
     def __repr__(self):
-        # This works as long as we accept all properties as paramters in the
-        # constructor
-        params = ['%s=%r' % (k, v) for k, v in self.__dict__.iteritems()]
-        return '%s(%s)' % (self.__class__.__name__,
-                           ', '.join(params))
+        return represent(self.__class__.__name__, self.__dict__)
 
     def sanitize(self):
         '''
@@ -121,8 +121,7 @@ class MapReplyRecord(object):
 
         # EID-prefix:  4 octets if an IPv4 address-family, 16 octets if an IPv6
         # address-family.
-        if not isinstance(self.eid_prefix, IP) \
-        or self.eid_prefix.version() not in (4, 6):
+        if not isinstance(self.eid_prefix, (IPv4Network, IPv6Network)):
             raise ValueError('EID prefix must be IPv4 or IPv6')
 
         # Check locator records
@@ -132,6 +131,13 @@ class MapReplyRecord(object):
 
             locator_record.sanitize()
 
+        # For each Map-Reply record, the list of Locators in a Locator-Set MUST
+        # appear in the same order for each ETR that originates a Map-Reply
+        # message.  The Locator-Set MUST be sorted in order of ascending IP
+        # address where an IPv4 locator address is considered numerically 'less
+        # than' an IPv6 locator address.
+        self.locator_records.sort(key=LocatorRecord.sort_key)
+
     @classmethod
     def from_bytes(cls, bitstream):
         '''
@@ -161,8 +167,8 @@ class MapReplyRecord(object):
         # Read the flag
         record.authoritative = bitstream.read('bool')
 
-        # Skip over reserved bits
-        bitstream.read(12 + 4)
+        # Read reserved bits
+        record._reserved1 = bitstream.read(12 + 4)
 
         # Read the map version
         record.map_version = bitstream.read('uint:12')
@@ -201,7 +207,7 @@ class MapReplyRecord(object):
         bitstream += BitArray('uint:8=%d' % len(self.locator_records))
 
         # Add the EID prefix mask length
-        bitstream += BitArray('uint:8=%d' % self.eid_prefix.prefixlen())
+        bitstream += BitArray('uint:8=%d' % self.eid_prefix.prefixlen)
 
         # Add the NMR action
         bitstream += BitArray('uint:3=%d' % self.action)
@@ -210,7 +216,7 @@ class MapReplyRecord(object):
         bitstream += BitArray('bool=%d' % self.authoritative)
 
         # Add reserved bits
-        bitstream += BitArray(12 + 4)
+        bitstream += self._reserved1
 
         # Add the map version
         bitstream += BitArray('uint:12=%d' % self.map_version)
