@@ -3,9 +3,9 @@ Created on 6 jan. 2013
 
 @author: sander
 '''
-from IPy import IP
 from bitstring import BitArray, ConstBitStream, Bits
-from pylisp.utils import make_prefix
+from ipaddress import IPv4Address, IPv6Address, IPv4Network, IPv6Network, \
+    ip_network
 
 
 def read_afi_address_from_bitstream(bitstream, prefix_len=None):
@@ -17,13 +17,13 @@ def read_afi_address_from_bitstream(bitstream, prefix_len=None):
     >>> afi_address = '0001c00002ab'.decode('hex')
     >>> bitstream = ConstBitStream(bytes=afi_address)
     >>> read_afi_address_from_bitstream(bitstream)
-    IP('192.0.2.171')
+    IPv4Address(u'192.0.2.171')
 
     If a prefix length is provided then a prefix is returned:
     >>> afi_address = '0001c0000200'.decode('hex')
     >>> bitstream = ConstBitStream(bytes=afi_address)
     >>> read_afi_address_from_bitstream(bitstream, 24)
-    IP('192.0.2.0/24')
+    IPv4Network(u'192.0.2.0/24')
 
     The function consumes the bits used by the AFI address from the
     bitstream, but won't read beyond that point:
@@ -50,16 +50,24 @@ def read_afi_address_from_bitstream(bitstream, prefix_len=None):
     elif afi == 1:
         # IPv4 address
         address_int = bitstream.read('uint:32')
-        address = IP(address_int, ipversion=4)
+        address = IPv4Address(address_int)
         if prefix_len is not None:
-            address = make_prefix(address, prefix_len)
+            orig_address = address
+            address = ip_network(address).supernet(new_prefix=prefix_len)
+            if address[0] != orig_address:
+                raise ValueError("invalid prefix length %s for %r"
+                                 % (prefix_len, address))
 
     elif afi == 2:
         # IPv6 address
         address_int = bitstream.read('uint:128')
-        address = IP(address_int, ipversion=6)
+        address = IPv6Address(address_int)
         if prefix_len is not None:
-            address = make_prefix(address, prefix_len)
+            orig_address = address
+            address = ip_network(address).supernet(new_prefix=prefix_len)
+            if address[0] != orig_address:
+                raise ValueError("invalid prefix length %s for %r"
+                                 % (prefix_len, address))
 
     elif afi == 16387:
         from pylisp.utils.lcaf import LCAFAddress
@@ -72,26 +80,30 @@ def read_afi_address_from_bitstream(bitstream, prefix_len=None):
 
 
 def get_bitstream_for_afi_address(address):
+    from pylisp.utils.lcaf import LCAFAddress
+
     # No address is AFI 0
     if address is None:
         return BitArray(16)
 
-    if isinstance(address, IP):
-        # IPv4
-        if address.version() == 4:
-            return BitArray('uint:16=1, uint:32=%d' % address.ip)
+    # IPv4
+    if isinstance(address, IPv4Address):
+        return BitArray('uint:16=1, uint:32=%d' % int(address))
 
-        # IPv6
-        if address.version() == 6:
-            return BitArray('uint:16=2, uint:128=%d' % address.ip)
+    elif isinstance(address, IPv4Network):
+        return BitArray('uint:16=1, uint:32=%d' % int(address[0]))
 
-        # Fall through
-        raise ValueError('Unsupported IP address version')
+    # IPv6
+    elif isinstance(address, IPv6Address):
+        return BitArray('uint:16=2, uint:128=%d' % int(address))
 
-    from pylisp.utils.lcaf import LCAFAddress
-    if isinstance(address, LCAFAddress):
+    elif isinstance(address, IPv6Network):
+        return BitArray('uint:16=2, uint:128=%d' % int(address[0]))
+
+    elif isinstance(address, LCAFAddress):
         address_bytes = bytes(address)
         return BitArray('uint:16=16387') + BitArray(bytes=address_bytes)
 
-    # Nobody encoded it...
-    raise ValueError('Unsupported address type')
+    else:
+        # Nobody encoded it...
+        raise ValueError('Unsupported address type')
