@@ -4,7 +4,7 @@ Created on 12 jan. 2013
 @author: sander
 '''
 from bitstring import BitArray
-from pylisp.utils import make_prefix
+from ipaddress import ip_network
 from pylisp.utils.afi import read_afi_address_from_bitstream, \
     get_bitstream_for_afi_address
 from pylisp.utils.lcaf import type_registry
@@ -18,8 +18,22 @@ class LCAFAFIListAddress(LCAFAddress):
         super(LCAFAFIListAddress, self).__init__()
         self.addresses = addresses or []
 
+    def __unicode__(self):
+        return u','.join(self.addresses)
+
+    def get_addresses(self):
+        addresses = []
+        for address in self.addresses:
+            if isinstance(address, LCAFAddress):
+                addresses += address.get_addresses()
+            else:
+                addresses.append(address)
+
+        return addresses
+
     def sanitize(self):
         super(LCAFAFIListAddress, self).sanitize()
+        # TODO: implement
 
     @classmethod
     def _from_data_bytes(cls, data, prefix_len=None, rsvd1=None, flags=None,
@@ -28,7 +42,11 @@ class LCAFAFIListAddress(LCAFAddress):
         while data.pos != data.len:
             address = read_afi_address_from_bitstream(data)
             if prefix_len is not None:
-                address = make_prefix(address, prefix_len)
+                orig_address = address
+                address = ip_network(address).supernet(new_prefix=prefix_len)
+                if address[0] != orig_address:
+                    raise ValueError("invalid prefix length %s for %r"
+                                     % (prefix_len, address))
             addresses.append(address)
         lcaf = cls(addresses=addresses)
         lcaf.sanitize()
