@@ -4,7 +4,7 @@ Created on 12 jan. 2013
 @author: sander
 '''
 from bitstring import BitArray
-from pylisp.utils import make_prefix
+from ipaddress import ip_network
 from pylisp.utils.afi import read_afi_address_from_bitstream, \
     get_bitstream_for_afi_address
 from pylisp.utils.lcaf import type_registry
@@ -19,8 +19,22 @@ class LCAFAutonomousSystemAddress(LCAFAddress):
         self.asn = asn
         self.address = address
 
+    def __unicode__(self):
+        return u'[AS{0}]{1}'.format(self.asn, self.address)
+
+    def get_addresses(self):
+        if isinstance(self.address, LCAFAddress):
+            return self.address.get_addresses()
+
+        return [self.address]
+
     def sanitize(self):
         super(LCAFAutonomousSystemAddress, self).sanitize()
+        # TODO: implement
+
+    @property
+    def prefixlen(self):
+        return self.address.prefixlen
 
     @classmethod
     def _from_data_bytes(cls, data, prefix_len=None, rsvd1=None, flags=None,
@@ -28,7 +42,11 @@ class LCAFAutonomousSystemAddress(LCAFAddress):
         asn = data.read('uint:32')
         address = read_afi_address_from_bitstream(data)
         if prefix_len is not None:
-            address = make_prefix(address, prefix_len)
+            orig_address = address
+            address = ip_network(address).supernet(new_prefix=prefix_len)
+            if address[0] != orig_address:
+                raise ValueError("invalid prefix length %s for %r"
+                                 % (prefix_len, address))
         lcaf = cls(asn=asn,
                    address=address)
         lcaf.sanitize()
