@@ -10,12 +10,17 @@ from pylisp.utils.lcaf.base import LCAFAddress
 
 class LCAFNullAddress(LCAFAddress):
     lcaf_type = 0
+    prefixlen = 0
 
     def __init__(self):
         super(LCAFNullAddress, self).__init__()
 
+    def __unicode__(self):
+        return u'NULL'
+
     def sanitize(self):
         super(LCAFNullAddress, self).sanitize()
+        # TODO: implement
 
     @classmethod
     def _from_data_bytes(cls, data, prefix_len=None, rsvd1=None, flags=None,
