@@ -1,6 +1,4 @@
 #!/usr/bin/env python
-from IPy import IP
-from unittests.utils import PacketTest, PacketTestCreator
 
 # Add the parent directory to the start of the path
 if __name__ == '__main__':
@@ -8,7 +6,9 @@ if __name__ == '__main__':
     sys.path.insert(0, '.')
     sys.path.insert(0, '..')
 
+from ipaddress import IPv4Address, IPv4Network
 from pylisp.packet.lisp.control import map_request
+from unittests.utils import PacketTest, PacketTestCreator
 import doctest
 import unittest
 
@@ -27,8 +27,8 @@ class MapRequestTestCase(unittest.TestCase):
     cases = [PacketTest(name='test_minimal_map_request',
                         desc='Generate a minimal map request',
                         cls=map_request.MapRequestMessage,
-                        params={'itr_rlocs': [IP('1.2.3.4')],
-                                'eid_prefixes': [IP('192.0.2.0/24')]},
+                        params={'itr_rlocs': [IPv4Address(u'1.2.3.4')],
+                                'eid_prefixes': [IPv4Network(u'192.0.2.0/24')]},
                         bytes_hex='1000000100000000000000000000'
                                   '00010102030400180001c0000200',
                         exception=(None, '')),
