@@ -45,21 +45,17 @@ def call_git_describe(abbrev=4, commit_hash=None):
         p = Popen(cmd, stdout=PIPE, stderr=PIPE)
         p.stderr.close()
         line = p.stdout.readlines()[0]
+        p.wait()
         return line.strip()
 
     except:
         return None
 
 
-def get_git_changelog(abbrev=4):
-    my_tag = call_git_describe(abbrev)
+def get_git_changelog(version, abbrev=4):
     prev_tag = None
     prev_tag_commit_hash = None
 
-    # Don't do changelogs for tags with a dash: they are commits between tags
-    if '-' in my_tag:
-        return None
-
     # Collect commits
     commits = []
 
@@ -73,6 +69,7 @@ def get_git_changelog(abbrev=4):
             p.stderr.close()
 
             commit_hash = p.stdout.readlines()[0].strip()
+            p.wait()
             if not commit_hash:
                 break
 
@@ -81,7 +78,7 @@ def get_git_changelog(abbrev=4):
                 break
 
             # Only look at tags without a dash, others are commits between tags
-            if '-' not in tag and tag != my_tag:
+            if '-' not in tag and tag != version:
                 prev_tag_commit_hash = commit_hash
                 prev_tag = tag
                 break
@@ -91,6 +88,8 @@ def get_git_changelog(abbrev=4):
             p = Popen(cmd, stdout=PIPE, stderr=PIPE)
             p.stderr.close()
             lines = p.stdout.readlines()
+            p.wait()
+
             lines = [l.strip() for l in lines]
             commits.append(lines)
 
@@ -105,9 +104,10 @@ def get_git_changelog(abbrev=4):
         p = Popen(cmd, stdout=PIPE, stderr=PIPE)
         p.stderr.close()
         when = p.stdout.readlines()[0].strip()
+        p.wait()
 
         # Build the changelog
-        header = 'Version %s' % my_tag
+        header = 'Version %s' % version
         changelog = [header,
                      '=' * len(header),
                      'Released: %s' % when,
@@ -141,20 +141,37 @@ def write_release_version(version):
     f.write("%s\n" % version)
     f.close()
 
+    # Add the changelog
+    cmd = ['git', 'add', 'RELEASE-VERSION']
+    p = Popen(cmd)
+    p.wait()
+
+    cmd = ['git', 'commit', '-m', 'Adding/updating RELEASE-VERSION']
+    p = Popen(cmd)
+    p.wait()
+
 
 def write_changelog(version, changelog):
-    f = open("changes/ChangeLog-%s.md" % version, "w")
+    filename = "changes/ChangeLog-%s.md" % version
+
+    f = open(filename, "w")
     f.write('%s\n' % changelog)
     f.close()
 
+    # Add the changelog
+    cmd = ['git', 'add', filename]
+    p = Popen(cmd)
+    p.wait()
 
-def get_git_version(abbrev=4):
+    cmd = ['git', 'commit', '-m', 'Adding/updating changelog']
+    p = Popen(cmd)
+    p.wait()
+
+
+def get_git_version(version=None, abbrev=4):
     # Read in the version that's currently in RELEASE-VERSION.
     release_version = read_release_version()
 
-    # First try to get the current version using “git describe”.
-    version = call_git_describe(abbrev)
-
     # If that doesn't work, fall back on the value that's in
     # RELEASE-VERSION.
     if version is None:
@@ -170,7 +187,7 @@ def get_git_version(abbrev=4):
         write_release_version(version)
 
     # Get the changelog
-    changelog = get_git_changelog()
+    changelog = get_git_changelog(version)
     if changelog:
         write_changelog(version, changelog)
 
@@ -179,4 +196,14 @@ def get_git_version(abbrev=4):
 
 
 if __name__ == "__main__":
-    print get_git_version()
+    import sys
+    version = get_git_version(sys.argv[1])
+    print version
+
+    cmd = ['git', 'tag', '-a', '-f', '-m', 'Tagging version %s' % version, version]
+    p = Popen(cmd)
+    p.wait()
+
+    cmd = ['git', 'push', '--tags']
+    p = Popen(cmd)
+    p.wait()
